# -*-python-*-

__package__    = "pocketMMap/Providers/__init__.py"
__version__    = "0.4"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/pocketMMap"
__date__       = "$Date: 2008/11/28 18:07:47 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import urllib2
from htmlentitydefs import name2codepoint as n2cp
import re
import pocketMMap

class Provider (pocketMMap.pocketMMap) :

    def fetch_url (self, url) :

        try :
            fh = urllib2.urlopen(url)
            return fh.read()
        except Exception, e :
            return None

    def calculate_bbox (self, points) :

        sw_lat = None
        sw_lon = None
        ne_lat = None
        ne_lon = None

        for pt in points :
            if not sw_lat or pt['lat'] < sw_lat :
                sw_lat = pt['lat']
                
            if not sw_lon or pt['lon'] < sw_lon :
                sw_lon = pt['lon']
                        
            if not ne_lat or pt['lat'] > ne_lat :
                ne_lat = pt['lat']

            if not ne_lon or pt['lon'] > ne_lon :
                ne_lon = pt['lon']

        return (sw_lat,sw_lon,ne_lat,ne_lon)

    def substitute_entity (self, match):
        ent = match.group(2)
        if match.group(1) == "#":
            return unichr(int(ent))
        else:
            cp = n2cp.get(ent)

            if cp:
                return unichr(cp)
            else:
                return match.group()

    def decode_htmlentities (self, string):
        entity_re = re.compile("&(#?)(\d{1,5}|\w{1,8});")
        return entity_re.subn(self.substitute_entity, string)[0]

class Atom (Provider) :

    def draw_feed (self, url, zoom=15) :

        xml = self.fetch_url(url)
        
        import elementtree.ElementTree as ET
        tree = ET.fromstring(xml)

        atom = 'http://www.w3.org/2005/Atom'

        points = []
        markers = []
        
        for entry in tree.findall(".//{%s}entry" % atom) : 

            pt = self.find_latlon(entry)

            if not pt :
                continue

            points.append(pt)

            # A dopplr-ism
            title = entry.findtext("{%s}title" % atom).split(" by ")[0]
            author = entry.findtext("{%s}author/{%s}name" % (atom, atom))

            tags = [author]
            
            for c in entry.findall("{%s}category" % atom) :
                tags.append(c.attrib['term'])
                
            markers.append({'label':title, 'latitude':pt['lat'], 'longitude':pt['lon'], 'tags':tags})

        bbox = self.calculate_bbox(points)
            
        self.add_markers(markers)
        self.draw(bbox, zoom)

    def find_latlon (self, entry) :

        gml = 'http://www.opengis.net/gml'
        atom = 'http://www.w3.org/2005/Atom'
        
        geo = entry.findtext(".//{%s}pos" % gml)
        
        if geo :
            pt = map(lambda x: float(x), geo.strip().split(" "))
            return {'lat':pt[0], 'lon':pt[1]}

        return None
        
class GeoRSS (Provider) :

    def draw_feed (self, url, zoom=15) :

        rss = self.fetch_url(url)
        
        from BeautifulSoup import BeautifulStoneSoup
        soup = BeautifulStoneSoup(rss)

        points = []
        markers = []

        p = re.compile("tag as ([^<]+)")
                
        for i in soup.findAll('item'):

            pt = self.find_latlon(i)
    
            if not pt :
                continue

            desc = i.find('description').contents[0]            
            tags = re.findall(p, desc)
            
            if len(tags) :
                tags = tags[0].split(" ")
                
                if "skip" in tags :
                    continue

            points.append(pt)
                    
            title = i.find('title').contents[0]
            title = self.decode_htmlentities(title)
                
            markers.append({'label':title,'latitude':pt['lat'],'longitude':pt['lon'], 'tags':tags})

        bbox = self.calculate_bbox(points)

        self.add_markers(markers)
        self.draw(bbox, zoom)

    def find_latlon (self, item) :

        geo = item.find('georss:point')

        if geo :
            pt = geo.contents[0]
            pt = map(lambda x: float(x), pt.strip().split(" "))
            return {'lat':pt[0], 'lon':pt[1]}

        return None
